from .constants import CONFIG_SYNC_API, CONFIG_SYNC_REQ_INPUT, PASSED, FAILED
from .utilities import generate_error_list, validate_input
from .logger import logger

import os
import time
import math
import json
import asyncio
        
def config_sync_check(central_api_req, input_data, **kwargs):
    """
    function to check Config Sync for the given devices
        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
                input_data (list(dict)): serial number of the device to check the config sync
                **kwargs: Additional or Optional arguments
                            limit (default 20): limit for concurrent executions for asyncio
                            time_sleep (default 20s): time sleep for each async execution

        Returns:
               list(dict): includes serial(str), status(Passed or Failed), ouput(str)
 

    """
    try:
        if len(input_data) > 0:
            if validate_input(CONFIG_SYNC_REQ_INPUT, input_data) != []:
                error_msg = ("Validation Failed for input list provided")
                return [{"serial": 'NA', "status": FAILED, "message": error_msg}]
            config_sync_output_list = []
            limit = kwargs.get('limit') if 'limit' in kwargs.keys() else 20
            for index in range(math.ceil(len(input_data)/limit)):
                subtasks = []
                for device in input_data[index*limit:(index+1)*limit]:
                    subtasks.append(configuration_sync_check(central_api_req, device.get("serial")))
                if os.name == "nt":
                    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
                config_sync_output_list.extend(asyncio.run(synctask(subtasks)))
                if len(input_data[index*limit:(index+1)*limit]) > 20:
                    logger.debug("Time sleep started")
                    time_sleep_in_secs = kwargs.get('time_sleep') if 'time_sleep' in kwargs.keys() else 20
                    time.sleep(time_sleep_in_secs)
                    logger.debug("Time sleep ended")
            return config_sync_output_list
        else:
            error_msg = "0 device selected to perform the task"
            logger.error(error_msg)
            return generate_error_list(input_data, FAILED, error_msg)
    except Exception as e:
        error_msg = f"Error while checking for config sync"
        logger.error(error_msg)
        return generate_error_list(input_data, FAILED, error_msg)


async def configuration_sync_check(central_api_req, serial):
    """
	Asynchronous function to assign group to a device

        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
				serial (str) : serial number of the selected device to check the config sync

        Returns:
                dict : includes serial(str), status(Passed or Failed), ouput(str)
	"""
    try:
        method = "GET"
        params={ "details": False }
        api_path = CONFIG_SYNC_API.format(serial)
        resp, resp_data = await central_api_req.asyncrequest(method, api_path, params=json.dumps(params))
        if resp.status == 404:
            error_msg = f"{resp.status} - Device not reachable on the central"
            logger.error(error_msg)
            return {"serial": serial, "status": FAILED, "message": error_msg}
        elif resp.status == 200:
            op = resp_data
            op = op[op.find('{'):op.find('}') + 1].replace("\\n", "")
            op = json.loads(op)
            config_sync_status = True if not op.get("Configuration_error_status") else False
            if config_sync_status:
                success_msg = f"{resp.status} - Configuration has got no error, In Sync"
                logger.info(success_msg)
                return {"serial": serial, "status": PASSED, "message": success_msg}
            else:
                error_msg = f"{resp.status} - configuration error"
                logger.error(error_msg)
                return {"serial": serial, "status": FAILED, "message": error_msg}
        else:
            error_msg = f"{resp.status} Error while checking config sync"
            logger.error(error_msg)
            return {"serial": serial, "status": FAILED, "message": error_msg}
    except Exception as e:
        error_msg = f"Error occurred while checking for the configuration sync"
        logger.error(error_msg)
        return {"serial": serial, "status": FAILED, "message": error_msg}

def template_sync_check(central_api_req, input_data, **kwargs):
    """
    function to check Template Sync for the given devices
        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
                input_data (list(dict)): serial number of the device to check the template sync
                **kwargs: Additional or Optional arguments
                            limit (default 20): limit for concurrent executions for asyncio
                            time_sleep (default 20s): time sleep for each async execution

        Returns:
               list(dict): includes serial(str), status(Passed or Failed), ouput(str)
 

    """
    try:
        if len(input_data) > 0:
            if validate_input(CONFIG_SYNC_REQ_INPUT, input_data) != []:
                error_msg = ("Validation Failed for input list provided")
                return [{"serial": 'NA', "status": FAILED, "message": error_msg}]
            template_sync_output_list = []
            limit = kwargs.get('limit') if 'limit' in kwargs.keys() else 20
            for index in range(math.ceil(len(input_data)/limit)):
                subtasks = []
                for device in input_data[index*limit:(index+1)*limit]:
                    subtasks.append(sync_check_template(central_api_req, device.get("serial")))
                if os.name == "nt":
                    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
                template_sync_output_list.extend(asyncio.run(synctask(subtasks)))
                if len(input_data[index*limit:(index+1)*limit]) > 20:
                    logger.debug("Time sleep started")
                    time_sleep_in_secs = kwargs.get('time_sleep') if 'time_sleep' in kwargs.keys() else 20
                    time.sleep(time_sleep_in_secs)
                    logger.debug("Time sleep ended")
            return template_sync_output_list
        else:
            error_msg = "0 device selected to perform the task"
            logger.error(error_msg)
            return generate_error_list(input_data, FAILED, error_msg)
    except Exception as e:
        error_msg = f"Error while checking for config sync"
        logger.error(error_msg)
        return generate_error_list(input_data, FAILED, error_msg)


async def sync_check_template(central_api_req, serial):
    """
	Asynchronous function to check Template Sync

        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
				serial (str) : serial number of the selected device to validate template sync

        Returns:
                dict : includes serial(str), status(Passed or Failed), ouput(str)
	"""
    try:
        method = "GET"
        params={ "details": False }
        api_path = CONFIG_SYNC_API.format(serial)
        resp, resp_data = await central_api_req.asyncrequest(method, api_path, params=json.dumps(params))
        if resp.status == 404:
            error_msg = f"{resp.status} - Device not reachable on the central"
            logger.error(error_msg)
            return {"serial": serial, "status": FAILED, "message": error_msg}
        elif resp.status == 200:
            op = resp_data
            op = op[op.find('{'):op.find('}') + 1].replace("\\n", "")
            op = json.loads(op)
            template_sync_status = True if not op.get("Template_error_status") else False
            if template_sync_status:
                success_msg = f"{serial} - Template has got no error, In Sync"
                logger.info(success_msg)
                return {"serial": serial, "status": PASSED, "message": success_msg}
            else:
                error_msg = f"Template error"
                logger.error(error_msg)
                return {"serial": serial, "status": FAILED, "message": error_msg}
        else:
            error_msg = f"{serial} Error while checking Template sync"
            logger.error(error_msg)
            return {"serial": serial, "status": FAILED, "message": error_msg}
    except Exception as e:
        error_msg = f"Error occurred while checking for the template sync"
        logger.error(error_msg)
        return {"serial": serial, "status": FAILED, "message": error_msg}

async def synctask(subtasks):
    task_level_output = await asyncio.gather(*subtasks)
    return task_level_output


